"""
Python client for Kubernetes
"""

__version__ = '0.18'

from .config import KubeConfig  # noqa
from .exceptions import KubernetesError, PyKubeError, ObjectDoesNotExist  # noqa
from .http import HTTPClient  # noqa
from .objects import (  # noqa
    object_factory,
    ConfigMap,
    CronJob,
    DaemonSet,
    Deployment,
    Endpoint,
    Event,
    HorizontalPodAutoscaler,
    Ingress,
    Job,
    LimitRange,
    Namespace,
    Node,
    PersistentVolume,
    PersistentVolumeClaim,
    Pod,
    PodDisruptionBudget,
    PodSecurityPolicy,
    ReplicationController,
    ReplicaSet,
    ResourceQuota,
    Secret,
    Service,
    ServiceAccount,
    StatefulSet,
    ThirdPartyResource,
    Role,
    ClusterRole,
    RoleBinding,
    ClusterRoleBinding,
)
from .query import now, all_ as all, everything  # noqa
